#!/usr/bin/perl -w

# rss/index.cgi
#
# This script is part of Birch. It displays the most recent posts as an RSS 2.0 feed.
#
# Copyright (c) 2006, Ross Harmes
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without modification, 
# are permitted provided that the following conditions are met:
# 
#	*	Redistributions of source code must retain the above copyright notice, 
#		this list of conditions and the following disclaimer.
#	*	Redistributions in binary form must reproduce the above copyright notice, 
#		this list of conditions and the following disclaimer in the documentation 
#		and/or other materials provided with the distribution.
#	*	Neither the name of View Software nor the names of its contributors may be 
#		used to endorse or promote products derived from this software without 
#		specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
# SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY 
# WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# Changelog: 
#	2005.06.19: File created.

use POSIX;

use strict;
use CGI qw(:standard);

my $photo_list 	 = "../cgi-bin/photos.xml"; 			# This is the XML file with the dates & captions of the posted photos.
my $title_file 	 = "../includes/title.shtml";
my $author_file	 = "../includes/author.shtml";
my $number_of_posts = 10;								# The number of posts you want displayed.
my $num_char = 500;										# The number of characters to display in the description.
my $generator_sig = 'Birch RSS Feed Generator v1.0';	# Signature of this script.

my ($dimensions, $date, $caption, $orientation, $year, $mon, $mday, $hour, $min, $sec, $weekday, $unix_time, $item_title, $pub_date, $image_tag);
my ($file_line, $sidebar, $site_URL, $site_title, $site_author, $current_year);
my $items = "";
my (@archive, @line);


my %month = ("00" => "", "01" => "Jan", "02" => "Feb", "03" => "Mar", "04" => "Apr", "05" => "May", "06" => "Jun", "07" => "Jul", "08" => "Aug", "09" => "Sep", "10" => "Oct", "11" => "Nov", "12" => "Dec");
my @wday = ("Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat");


# Get the site info: URL, title, copyright name and current year.

$site_URL = "http://" . $ENV{SERVER_NAME};
$site_URL = CGI::url();

open TITLE, $title_file or die "Can't open $title_file for reading: $!";
while ($file_line = <TITLE>) { $site_title .= $file_line; }
close TITLE;

open AUTHOR, $author_file or die "Can't open $author_file for reading: $!";
while ($file_line = <AUTHOR>) { $site_author .= $file_line; }
close AUTHOR;

$current_year = `date "+%Y"`;

# Open the photo list and read the lines into an array in reverse order.

open(LIST, "< $photo_list");
@line = <LIST>;
close LIST;

# Process the array and display the most recent posts.

for(my $n = 3; $n < scalar(@line) && $n < $number_of_posts + 4; $n++) {
	if(defined $line[$n] && $line[$n] =~ /<date>(.*?)<\/date><caption>(.*?)<\/caption><orientation>(.*?)<\/orientation>/) {
	
		$date = $1;
		$caption = $2;
		$orientation = $3;
		
		# Process the date.
		
		$date =~ /(\d\d\d\d)_(\d\d)_(\d\d)_(\d\d)_(\d\d)_(\d\d)/;
		
		$year 	= $1;
		$mon 	= $2;
		$mday 	= $3;
		$hour 	= $4;
		$min 	= $5;
		$sec 	= $6;
		
		$unix_time = mktime($sec, $min, $hour - 1, $mday, $mon - 1, $year - 1900, 0, 0);
		
		$weekday = $wday[(localtime $unix_time)[6]];
		$mon = $month{$2};
		
		$item_title = "Photo posted on $mon $mday, $year at $hour:$min";
		$pub_date = "$weekday, $mday $mon $year $hour:$min:$sec EST";
		
		# Create the image tag.
		
		$image_tag = "<img src=\"$site_URL../photos/$date.jpg\" alt=\"\" ";
		$image_tag .= ($orientation eq 'horizontal') ? "width=\"600\" height=\"450\" />" : "width=\"450\" height=\"600\" />";

		# Pull out all of the XHTML tags.
	
		$caption =~ s/<(.|\s)*?>//g;
		
		# Reduce the string to $num_char of characters.
		
		$caption = (length($caption) <= $num_char) ? $caption : substr($caption, 0, $num_char) . "...\n";
		
		# Assemble it together into an <item>.
        
		$items .= 	"		<item>\n" .
					"			<title>$item_title</title>\n" .
					"			<link>$site_URL../cgi-bin/frame.cgi?date=$date</link>\n" .
					"			<description>$image_tag\n$caption</description>\n" .
					"			<pubDate>$pub_date</pubDate>\n" .
					"		</item>\n";
		}
	}

# Print the feed. 

print <<END_OF_XML
Content-type: text/html

<?xml version="1.0" encoding="utf-8"?>
<rss version="2.0">
	<channel>
		<title>$site_title</title>
		<link>$site_URL</link>
		<description>RSS 2.0 feed for $site_title</description>
		<language>en-us</language>
		<copyright>Copyright $current_year $site_author</copyright>
		<generator>$generator_sig</generator>

$items

	</channel>
</rss>

END_OF_XML
;
